@@ -5,13 +5,22 @@
 
 #define MAIN_PRIVATE
 #include "or.h"
+#include "config.h"
+#include "main.h"
+#include "ntmain.h"
 
-#include <tchar.h>
-#define GENSRV_SERVICENAME  TEXT("tor")
-#define GENSRV_DISPLAYNAME  TEXT("Tor Win32 Service")
+#ifdef HAVE_EVENT2_EVENT_H
+#include <event2/event.h>
+#else
+#include <event.h>
+#endif
+
+#include <windows.h>
+#define GENSRV_SERVICENAME  "tor"
+#define GENSRV_DISPLAYNAME  "Tor Win32 Service"
 #define GENSRV_DESCRIPTION  \
-  TEXT("Provides an anonymous Internet communication system")
-#define GENSRV_USERACCT TEXT("NT AUTHORITY\\LocalService")
+  "Provides an anonymous Internet communication system"
+#define GENSRV_USERACCT "NT AUTHORITY\\LocalService"
 
 // Cheating: using the pre-defined error codes, tricks Windows into displaying
 //           a semi-related human-readable error message if startup fails as
@@ -27,7 +36,6 @@ static SERVICE_STATUS_HANDLE hStatus;
  * to the NT service functions. */
 static char **backup_argv;
 static int backup_argc;
-static char* nt_strerror(uint32_t errnum);
 
 static void nt_service_control(DWORD request);
 static void nt_service_body(int argc, char **argv);
@@ -46,6 +54,11 @@ static int nt_service_cmd_stop(void);
 struct service_fns {
   int loaded;
 
+  /** @{ */
+  /** Function pointers for Windows API functions related to service
+   * management.  These are NULL, or they point to the .  They're set by
+   * calling the LOAD macro below.  */
+
   BOOL (WINAPI *ChangeServiceConfig2A_fn)(
                              SC_HANDLE hService,
                              DWORD dwInfoLevel,
@@ -61,30 +74,30 @@ struct service_fns {
 
   SC_HANDLE (WINAPI *CreateServiceA_fn)(
                              SC_HANDLE hSCManager,
-                             LPCTSTR lpServiceName,
-                             LPCTSTR lpDisplayName,
+                             LPCSTR lpServiceName,
+                             LPCSTR lpDisplayName,
                              DWORD dwDesiredAccess,
                              DWORD dwServiceType,
                              DWORD dwStartType,
                              DWORD dwErrorControl,
-                             LPCTSTR lpBinaryPathName,
-                             LPCTSTR lpLoadOrderGroup,
+                             LPCSTR lpBinaryPathName,
+                             LPCSTR lpLoadOrderGroup,
                              LPDWORD lpdwTagId,
-                             LPCTSTR lpDependencies,
-                             LPCTSTR lpServiceStartName,
-                             LPCTSTR lpPassword);
+                             LPCSTR lpDependencies,
+                             LPCSTR lpServiceStartName,
+                             LPCSTR lpPassword);
 
   BOOL (WINAPI *DeleteService_fn)(
                              SC_HANDLE hService);
 
   SC_HANDLE (WINAPI *OpenSCManagerA_fn)(
-                             LPCTSTR lpMachineName,
-                             LPCTSTR lpDatabaseName,
+                             LPCSTR lpMachineName,
+                             LPCSTR lpDatabaseName,
                              DWORD dwDesiredAccess);
 
   SC_HANDLE (WINAPI *OpenServiceA_fn)(
                              SC_HANDLE hSCManager,
-                             LPCTSTR lpServiceName,
+                             LPCSTR lpServiceName,
                              DWORD dwDesiredAccess);
 
   BOOL (WINAPI *QueryServiceStatus_fn)(
@@ -92,28 +105,29 @@ struct service_fns {
                              LPSERVICE_STATUS lpServiceStatus);
 
   SERVICE_STATUS_HANDLE (WINAPI *RegisterServiceCtrlHandlerA_fn)(
-                             LPCTSTR lpServiceName,
+                             LPCSTR lpServiceName,
                              LPHANDLER_FUNCTION lpHandlerProc);
 
   BOOL (WINAPI *SetServiceStatus_fn)(SERVICE_STATUS_HANDLE,
                              LPSERVICE_STATUS);
 
   BOOL (WINAPI *StartServiceCtrlDispatcherA_fn)(
-                             const SERVICE_TABLE_ENTRY* lpServiceTable);
+                             const SERVICE_TABLE_ENTRYA* lpServiceTable);
 
   BOOL (WINAPI *StartServiceA_fn)(
                              SC_HANDLE hService,
                              DWORD dwNumServiceArgs,
-                             LPCTSTR* lpServiceArgVectors);
+                             LPCSTR* lpServiceArgVectors);
 
   BOOL (WINAPI *LookupAccountNameA_fn)(
-                             LPCTSTR lpSystemName,
-                             LPCTSTR lpAccountName,
+                             LPCSTR lpSystemName,
+                             LPCSTR lpAccountName,
                              PSID Sid,
                              LPDWORD cbSid,
                              LPTSTR ReferencedDomainName,
                              LPDWORD cchReferencedDomainName,
                              PSID_NAME_USE peUse);
+  /** @} */
 } service_fns = { 0,
                   NULL, NULL, NULL, NULL, NULL, NULL,
                   NULL, NULL, NULL, NULL, NULL, NULL,
@@ -130,13 +144,16 @@ nt_service_loadlibrary(void)
   if (service_fns.loaded)
     return;
 
-  /* XXXX Possibly, we should hardcode the location of this DLL. */
-  if (!(library = LoadLibrary("advapi32.dll"))) {
+  if (!(library = load_windows_system_library(TEXT("advapi32.dll")))) {
     log_err(LD_GENERAL, "Couldn't open advapi32.dll.  Are you trying to use "
             "NT services on Windows 98? That doesn't work.");
     goto err;
   }
 
+/* Helper macro: try to load a function named <b>f</b> from "library" into
+ * service_functions.<b>f</b>_fn.  On failure, log an error message, and goto
+ * err.
+ */
 #define LOAD(f) STMT_BEGIN                                              \
     if (!(fn = GetProcAddress(library, #f))) {                          \
       log_err(LD_BUG,                                                   \
@@ -218,7 +235,7 @@ nt_service_control(DWORD request)
           log_notice(LD_GENERAL,
                      "Got stop/shutdown request; shutting down cleanly.");
           service_status.dwCurrentState = SERVICE_STOP_PENDING;
-          event_loopexit(&exit_now);
+          event_base_loopexit(tor_libevent_get_base(), &exit_now);
           return;
   }
   service_fns.SetServiceStatus_fn(hStatus, &service_status);
@@ -275,20 +292,20 @@ nt_service_body(int argc, char **argv)
 static void
 nt_service_main(void)
 {
-  SERVICE_TABLE_ENTRY table[2];
+  SERVICE_TABLE_ENTRYA table[2];
   DWORD result = 0;
   char *errmsg;
   nt_service_loadlibrary();
   table[0].lpServiceName = (char*)GENSRV_SERVICENAME;
-  table[0].lpServiceProc = (LPSERVICE_MAIN_FUNCTION)nt_service_body;
+  table[0].lpServiceProc = (LPSERVICE_MAIN_FUNCTIONA)nt_service_body;
   table[1].lpServiceName = NULL;
   table[1].lpServiceProc = NULL;
 
   if (!service_fns.StartServiceCtrlDispatcherA_fn(table)) {
     result = GetLastError();
-    errmsg = nt_strerror(result);
+    errmsg = format_win32_error(result);
     printf("Service error %d : %s\n", (int) result, errmsg);
-    LocalFree(errmsg);
+    tor_free(errmsg);
     if (result == ERROR_FAILED_SERVICE_CONTROLLER_CONNECT) {
       if (tor_init(backup_argc, backup_argv) < 0)
         return;
@@ -323,9 +340,9 @@ nt_service_open_scm(void)
   nt_service_loadlibrary();
   if ((hSCManager = service_fns.OpenSCManagerA_fn(
                             NULL, NULL, SC_MANAGER_CREATE_SERVICE)) == NULL) {
-    errmsg = nt_strerror(GetLastError());
+    errmsg = format_win32_error(GetLastError());
     printf("OpenSCManager() failed : %s\n", errmsg);
-    LocalFree(errmsg);
+    tor_free(errmsg);
   }
   return hSCManager;
 }
@@ -340,9 +357,9 @@ nt_service_open(SC_HANDLE hSCManager)
   nt_service_loadlibrary();
   if ((hService = service_fns.OpenServiceA_fn(hSCManager, GENSRV_SERVICENAME,
                               SERVICE_ALL_ACCESS)) == NULL) {
-    errmsg = nt_strerror(GetLastError());
+    errmsg = format_win32_error(GetLastError());
     printf("OpenService() failed : %s\n", errmsg);
-    LocalFree(errmsg);
+    tor_free(errmsg);
   }
   return hService;
 }
@@ -374,14 +391,14 @@ nt_service_start(SC_HANDLE hService)
       printf("Service started successfully\n");
       return 0;
     } else {
-      errmsg = nt_strerror(service_status.dwWin32ExitCode);
+      errmsg = format_win32_error(service_status.dwWin32ExitCode);
       printf("Service failed to start : %s\n", errmsg);
-      LocalFree(errmsg);
+      tor_free(errmsg);
     }
   } else {
-    errmsg = nt_strerror(GetLastError());
+    errmsg = format_win32_error(GetLastError());
     printf("StartService() failed : %s\n", errmsg);
-    LocalFree(errmsg);
+    tor_free(errmsg);
   }
   return -1;
 }
@@ -418,14 +435,14 @@ nt_service_stop(SC_HANDLE hService)
     } else if (wait_time == MAX_SERVICE_WAIT_TIME) {
       printf("Service did not stop within %d seconds.\n", wait_time);
     } else {
-      errmsg = nt_strerror(GetLastError());
+      errmsg = format_win32_error(GetLastError());
       printf("QueryServiceStatus() failed : %s\n",errmsg);
-      LocalFree(errmsg);
+      tor_free(errmsg);
     }
   } else {
-    errmsg = nt_strerror(GetLastError());
+    errmsg = format_win32_error(GetLastError());
     printf("ControlService() failed : %s\n", errmsg);
-    LocalFree(errmsg);
+    tor_free(errmsg);
   }
   return -1;
 }
@@ -439,6 +456,7 @@ static char *
 nt_service_command_line(int *using_default_torrc)
 {
   TCHAR tor_exe[MAX_PATH+1];
+  char tor_exe_ascii[MAX_PATH+1];
   char *command, *options=NULL;
   smartlist_t *sl;
   int i, cmdlen;
@@ -464,18 +482,25 @@ nt_service_command_line(int *using_default_torrc)
     options = smartlist_join_strings(sl,"\" \"",0,NULL);
   smartlist_free(sl);
 
+#ifdef UNICODE
+  wcstombs(tor_exe_ascii, tor_exe, sizeof(tor_exe_ascii));
+#else
+  strlcpy(tor_exe_ascii, tor_exe, sizeof(tor_exe_ascii));
+#endif
+
   /* Allocate a string for the NT service command line */
-  cmdlen = strlen(tor_exe) + (options?strlen(options):0) + 32;
+  cmdlen = strlen(tor_exe_ascii) + (options?strlen(options):0) + 32;
   command = tor_malloc(cmdlen);
 
   /* Format the service command */
   if (options) {
     if (tor_snprintf(command, cmdlen, "\"%s\" --nt-service \"%s\"",
-                     tor_exe, options)<0) {
+                     tor_exe_ascii, options)<0) {
       tor_free(command); /* sets command to NULL. */
     }
   } else { /* ! options */
-    if (tor_snprintf(command, cmdlen, "\"%s\" --nt-service", tor_exe)<0) {
+    if (tor_snprintf(command, cmdlen, "\"%s\" --nt-service",
+                     tor_exe_ascii)<0) {
       tor_free(command); /* sets command to NULL. */
     }
   }
@@ -500,10 +525,10 @@ nt_service_install(int argc, char **argv)
 
   SC_HANDLE hSCManager = NULL;
   SC_HANDLE hService = NULL;
-  SERVICE_DESCRIPTION sdBuff;
+  SERVICE_DESCRIPTIONA sdBuff;
   char *command;
   char *errmsg;
-  const char *user_acct = GENSRV_USERACCT;
+  const char *user_acct = NULL;
   const char *password = "";
   int i;
   OSVERSIONINFOEX info;
@@ -547,13 +572,12 @@ nt_service_install(int argc, char **argv)
       is_win2k_or_worse = 1;
   }
 
-  if (user_acct == GENSRV_USERACCT) {
+  if (!user_acct) {
     if (is_win2k_or_worse) {
       /* On Win2k, there is no LocalService account, so we actually need to
        * fall back on NULL (the system account). */
       printf("Running on Win2K or earlier, so the LocalService account "
              "doesn't exist.  Falling back to SYSTEM account.\n");
-      user_acct = NULL;
     } else {
       /* Genericity is apparently _so_ last year in Redmond, where some
        * accounts are accounts that you can look up, and some accounts
@@ -562,6 +586,7 @@ nt_service_install(int argc, char **argv)
        */
       printf("Running on a Post-Win2K OS, so we'll assume that the "
              "LocalService account exists.\n");
+      user_acct = GENSRV_USERACCT;
     }
   } else if (0 && service_fns.LookupAccountNameA_fn(NULL, // On this system
                             user_acct,
@@ -590,10 +615,10 @@ nt_service_install(int argc, char **argv)
                                 SERVICE_AUTO_START, SERVICE_ERROR_IGNORE,
                                 command, NULL, NULL, NULL,
                                 user_acct, password)) == NULL) {
-    errmsg = nt_strerror(GetLastError());
+    errmsg = format_win32_error(GetLastError());
     printf("CreateService() failed : %s\n", errmsg);
     service_fns.CloseServiceHandle_fn(hSCManager);
-    LocalFree(errmsg);
+    tor_free(errmsg);
     tor_free(command);
     return -1;
   }
@@ -634,9 +659,9 @@ nt_service_remove(void)
 
   nt_service_stop(hService);
   if (service_fns.DeleteService_fn(hService) == FALSE) {
-    errmsg = nt_strerror(GetLastError());
+    errmsg = format_win32_error(GetLastError());
     printf("DeleteService() failed : %s\n", errmsg);
-    LocalFree(errmsg);
+    tor_free(errmsg);
     service_fns.CloseServiceHandle_fn(hService);
     service_fns.CloseServiceHandle_fn(hSCManager);
     return -1;
@@ -693,20 +718,6 @@ nt_service_cmd_stop(void)
   return stop;
 }
 
-/** Given a Win32 error code, this attempts to make Windows
- * return a human-readable error message. The char* returned
- * is allocated by Windows, but should be freed with LocalFree()
- * when finished with it. */
-static char*
-nt_strerror(uint32_t errnum)
-{
-   char *msgbuf;
-   FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
-                 NULL, errnum, MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
-                 (LPSTR)&msgbuf, 0, NULL);
-   return msgbuf;
-}
-
 int
 nt_service_parse_options(int argc, char **argv, int *should_exit)
 {
